
{*****************************************}
{                                         }
{             FastReport v2.2             }
{           Rich Add-In Object            }
{                                         }
{  Copyright (c) 1998-99 by Tzyganenko A. }
{                                         }
{*****************************************}

unit FR_Rich;

interface

{$I FR_Vers.inc}

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Buttons, ExtCtrls, ComCtrls, ClipBrd, Spin,
  DB, {$IFDEF Delphi2} DBTables,{$ENDIF} FR_Class, RichEdit, FR_Ctrls;

type
  TfrRichObject = class(TComponent)  // fake component
  end;

  TfrRichView = class(TfrView)
  public
    RichEdit: TRichEdit;
    constructor Create; override;
    procedure Assign(From:TfrView); override;
    destructor Destroy; override;
    procedure Draw(Canvas:TCanvas); override;
    procedure LoadFromStream(Stream:TStream); override;
    procedure SaveToStream(Stream:TStream); override;
    procedure GetBlob(b:TField); override;
  end;

  TRichForm = class(TfrObjEditorForm)
    OpenDialog: TOpenDialog;
    SaveDialog: TSaveDialog;
    SpeedBar: TPanel;
    OpenButton: TfrSpeedButton;
    SaveButton: TfrSpeedButton;
    UndoButton: TfrSpeedButton;
    Ruler: TPanel;
    FontDialog1: TFontDialog;
    FirstInd: TLabel;
    LeftInd: TLabel;
    RulerLine: TBevel;
    RightInd: TLabel;
    BoldButton: TfrSpeedButton;
    FontName: TComboBox;
    ItalicButton: TfrSpeedButton;
    LeftAlign: TfrSpeedButton;
    CenterAlign: TfrSpeedButton;
    RightAlign: TfrSpeedButton;
    UnderlineButton: TfrSpeedButton;
    BulletsButton: TfrSpeedButton;
    RichEdit1: TRichEdit;
    SpeedButton1: TfrSpeedButton;
    CancBtn: TfrSpeedButton;
    OkBtn: TfrSpeedButton;
    SpeedButton2: TfrSpeedButton;
    Image1: TImage;
    Bevel1: TBevel;
    Edit1: TEdit;
    FontSize: TUpDown;

    procedure SelectionChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FileOpen(Sender: TObject);
    procedure FileSaveAs(Sender: TObject);
    procedure EditUndo(Sender: TObject);
    procedure SelectFont(Sender: TObject);
    procedure RulerResize(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure BoldButtonClick(Sender: TObject);
    procedure FontSizeChange(Sender: TObject);
    procedure AlignButtonClick(Sender: TObject);
    procedure FontNameChange(Sender: TObject);
    procedure BulletsButtonClick(Sender: TObject);
    procedure RulerItemMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure RulerItemMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure FirstIndMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure LeftIndMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure RightIndMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure CancBtnClick(Sender: TObject);
    procedure OkBtnClick(Sender: TObject);
    procedure SpeedButton2Click(Sender: TObject);
    procedure FormActivate(Sender: TObject);
  private
    FUpdating: Boolean;
    FDragOfs: Integer;
    FDragging: Boolean;
    function CurrText: TTextAttributes;
    procedure GetFontNames;
    procedure SetupRuler;
    procedure SetEditRect;
  public
    procedure ShowEditor(t:TfrView); override;
  end;

var
  RichForm: TRichForm;

implementation

uses FR_Var, FR_Pars, FR_Utils;

const
  RulerAdj = 4/3;
  GutterWid = 6;

{$R *.DFM}


var
  SRichEdit: TRichEdit; // temporary rich used during TRichView drawing

procedure AssignRich(Rich1,Rich2: TRichEdit);
var st: TMemoryStream;
begin
  st := TMemoryStream.Create;
  Rich2.Lines.SaveToStream(st);
  st.Position := 0;
  Rich1.Lines.LoadFromStream(st);
  st.Free;
end;

{----------------------------------------------------------------------------}
constructor TfrRichView.Create;
begin
  inherited Create;
  RichEdit := TRichEdit.Create(nil);
  RichEdit.Parent := RichForm;
  RichEdit.Visible := False;
end;

procedure TfrRichView.Assign(From:TfrView);
begin
  inherited Assign(From);
  AssignRich(RichEdit,(From as TfrRichView).RichEdit);
end;

destructor TfrRichView.Destroy;
begin
  if RichForm <> nil then RichEdit.Free;
  inherited Destroy;
end;

procedure TfrRichView.Draw(Canvas:TCanvas);
var i: Integer;
    re: TRichEdit;
procedure ShowRich;
var
  Range: TFormatRange;
  LastChar, MaxLen, LogX: Integer;
begin
  FillChar(Range, SizeOf(TFormatRange), 0);
  with Range do
  begin
    LogX := Screen.PixelsPerInch;
    hdc := Canvas.Handle;
    hdcTarget := hdc;
    rc := Rect(DRect.Left * 1440 div LogX, DRect.Top * 1440 div LogX,
               DRect.Right * 1440 div LogX, DRect.Bottom * 1440 div LogX);
    rcPage := rc;
    LastChar := 0;
    MaxLen := re.GetTextLen;
    chrg.cpMax := -1;
    repeat
      chrg.cpMin := LastChar;
      LastChar := re.Perform(EM_FORMATRANGE, 1, Longint(@Range));
    until (LastChar >= MaxLen) or (LastChar = -1) or (LastChar = 0);
  end;
  re.Perform(EM_FORMATRANGE, 0, 0);
end;

procedure GetRichData(lnum:Integer);
var i,j,sum: Integer;
    s,s1,s2: String;
begin
  s := re.Lines[lnum];
  sum := 0;
  for i := 0 to lnum-1 do
  begin
    j := Length(re.Lines[i]);
    sum := sum + j + 2;
  end;
  i := 1;
  repeat
    while (i < Length(s)) and (s[i] <> '[') do Inc(i);
    s1 := GetBrackedVariable(s,i,j);
    if i <> j then
    begin
      Delete(s,i,j-i+1);
      s2 := '';
      CurReport.InternalOnGetValue(s1,s2);
      re.SelStart := sum + i;
      re.SelLength := j-i+1;
      ClipBoard.AsText := s2;
      re.PasteFromClipboard;
      Insert(s2,s,i);
      Inc(i,Length(s2));
    end;
  until i = j;
end;

begin
  BeginDraw(Canvas);
  if DocMode = dmPrinting then
  begin
    Memo1.Assign(Memo);
    CurReport.InternalOnEnterRect(Memo1,Self);
  end;
  CalcGaps;
  with Canvas do
  begin
    ShowBackground;
    re := RichEdit;
    if DocMode = dmPrinting then
    begin
      re := SRichEdit;
      AssignRich(re,RichEdit);
      for i := 0 to re.Lines.Count-1 do
        GetRichData(i);
    end;
    Brush.Style := bsClear;
    ShowRich;
    ShowFrame;
  end;
  if DocMode = dmPrinting then CurReport.InternalOnExportData(Self);
end;

procedure TfrRichView.LoadFromStream(Stream:TStream);
var b: Byte;
    n: Integer;
begin
  inherited LoadFromStream(Stream);
  Stream.Read(b,1);
  Stream.Read(n,4);
  if b <> 0 then RichEdit.Lines.LoadFromStream(Stream);
  Stream.Seek(n,soFromBeginning);
end;

procedure TfrRichView.SaveToStream(Stream:TStream);
var b: Byte;
    n,o: Integer;
begin
  inherited SaveToStream(Stream);
  b := 0;
  if RichEdit.Lines.Count <> 0 then b := 1;
  Stream.Write(b,1);
  n := Stream.Position;
  Stream.Write(n,4);
  if b <> 0 then RichEdit.Lines.SaveToStream(Stream);
  o := Stream.Position;
  Stream.Seek(n,soFromBeginning);
  Stream.Write(o,4);
  Stream.Seek(0,soFromEnd);
end;

procedure TfrRichView.GetBlob(b:TField);
var s: TMemoryStream;
begin
  s := TMemoryStream.Create;
  (b as TBlobField).SaveToStream(s);
  s.Position := 0;
  RichEdit.Lines.LoadFromStream(s);
  s.Free;
end;








{------------------------------------------------------------------------}
procedure TRichForm.ShowEditor(t:TfrView);
begin
  AssignRich(RichEdit1,(t as TfrRichView).RichEdit);
  if ShowModal = mrOk then AssignRich((t as TfrRichView).RichEdit,RichEdit1);
  RichEdit1.Lines.Clear;
end;

procedure TRichForm.SelectionChange(Sender: TObject);
begin
  with RichEdit1.Paragraph do
  try
    FUpdating := True;
    FirstInd.Left := Trunc(FirstIndent*RulerAdj)-4+GutterWid;
    LeftInd.Left := Trunc((LeftIndent+FirstIndent)*RulerAdj)-4+GutterWid;
    RightInd.Left := Ruler.ClientWidth-6-Trunc((RightIndent+GutterWid)*RulerAdj);
    BoldButton.Down := fsBold in RichEdit1.SelAttributes.Style;
    ItalicButton.Down := fsItalic in RichEdit1.SelAttributes.Style;
    UnderlineButton.Down := fsUnderline in RichEdit1.SelAttributes.Style;
    BulletsButton.Down := Boolean(Numbering);
    FontSize.Position := RichEdit1.SelAttributes.Size;
    FontName.Text := RichEdit1.SelAttributes.Name;
    case Ord(Alignment) of
      0: LeftAlign.Down := True;
      1: RightAlign.Down := True;
      2: CenterAlign.Down := True;
    end;
  finally
    FUpdating := False;
  end;
end;

function TRichForm.CurrText: TTextAttributes;
begin
  if RichEdit1.SelLength > 0 then
    Result := RichEdit1.SelAttributes
  else
    Result := RichEdit1.DefAttributes;
end;

procedure TRichForm.GetFontNames;
begin
  FontName.Items := Screen.Fonts;
  FontName.Sorted := True;
end;

procedure TRichForm.SetupRuler;
var
  I: Integer;
  S: String;
begin
  SetLength(S, 201);
  I := 1;
  while I < 200 do
  begin
    S[I] := #9;
    S[I+1] := '|';
    Inc(I, 2);
  end;
  Ruler.Caption := S;
end;

procedure TRichForm.SetEditRect;
var
  R: TRect;
begin
  with RichEdit1 do
  begin
    R := Rect(GutterWid, 0, ClientWidth-GutterWid, ClientHeight);
    SendMessage(Handle, EM_SETRECT, 0, Longint(@R));
  end;
end;

{ Event Handlers }

procedure TRichForm.FormCreate(Sender: TObject);
begin
  OpenDialog.InitialDir := ExtractFilePath(ParamStr(0));
  SaveDialog.InitialDir := OpenDialog.InitialDir;
  GetFontNames;
  SetupRuler;
  SelectionChange(Self);
end;

procedure TRichForm.FormResize(Sender: TObject);
begin
  SetEditRect;
  SelectionChange(Sender);
end;

procedure TRichForm.FormPaint(Sender: TObject);
begin
  SetEditRect;
end;

procedure TRichForm.FileOpen(Sender: TObject);
begin
  OpenDialog.Filter := frLoadStr('SRTFFile') + ' (*.rtf)|*.rtf';
  if OpenDialog.Execute then
  begin
    RichEdit1.Lines.LoadFromFile(OpenDialog.FileName);
    RichEdit1.SetFocus;
    SelectionChange(Self);
  end;
end;

procedure TRichForm.FileSaveAs(Sender: TObject);
begin
  SaveDialog.Filter := frLoadStr('SRTFFile') + ' (*.rtf)|*.rtf|' +
                       frLoadStr('STextFile') + ' (*.txt)|*.txt';
  if SaveDialog.Execute then
    RichEdit1.Lines.SaveToFile(SaveDialog.FileName);
end;

procedure TRichForm.EditUndo(Sender: TObject);
begin
  with RichEdit1 do
    if HandleAllocated then SendMessage(Handle, EM_UNDO, 0, 0);
end;

procedure TRichForm.SelectFont(Sender: TObject);
begin
  FontDialog1.Font.Assign(RichEdit1.SelAttributes);
  if FontDialog1.Execute then
    CurrText.Assign(FontDialog1.Font);
  RichEdit1.SetFocus;
end;

procedure TRichForm.RulerResize(Sender: TObject);
begin
  RulerLine.Width := Ruler.ClientWidth - (RulerLine.Left*2);
end;

procedure TRichForm.BoldButtonClick(Sender: TObject);
var s: TFontStyles;
begin
  if FUpdating then Exit;
  s := [];
  if BoldButton.Down then s := s + [fsBold];
  if ItalicButton.Down then s := s + [fsItalic];
  if UnderlineButton.Down then s := s + [fsUnderline];
  CurrText.Style := s;
end;

procedure TRichForm.FontSizeChange(Sender: TObject);
begin
  if FUpdating then Exit;
  CurrText.Size := FontSize.Position;
end;

procedure TRichForm.AlignButtonClick(Sender: TObject);
begin
  if FUpdating then Exit;
  RichEdit1.Paragraph.Alignment := TAlignment(TControl(Sender).Tag);
end;

procedure TRichForm.FontNameChange(Sender: TObject);
begin
  if FUpdating then Exit;
  CurrText.Name := FontName.Items[FontName.ItemIndex];
end;

procedure TRichForm.BulletsButtonClick(Sender: TObject);
begin
  if FUpdating then Exit;
  RichEdit1.Paragraph.Numbering := TNumberingStyle(BulletsButton.Down);
end;

{ Ruler Indent Dragging }

procedure TRichForm.RulerItemMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  FDragOfs := (TLabel(Sender).Width div 2);
  TLabel(Sender).Left := TLabel(Sender).Left+X-FDragOfs;
  FDragging := True;
end;

procedure TRichForm.RulerItemMouseMove(Sender: TObject; Shift: TShiftState;
  X, Y: Integer);
begin
  if FDragging then
    TLabel(Sender).Left :=  TLabel(Sender).Left+X-FDragOfs
end;

procedure TRichForm.FirstIndMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  FDragging := False;
  RichEdit1.Paragraph.FirstIndent := Trunc((FirstInd.Left+FDragOfs-GutterWid) / RulerAdj);
  LeftIndMouseUp(Sender, Button, Shift, X, Y);
end;

procedure TRichForm.LeftIndMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  FDragging := False;
  RichEdit1.Paragraph.LeftIndent := Trunc((LeftInd.Left+FDragOfs-GutterWid) / RulerAdj)-RichEdit1.Paragraph.FirstIndent;
  SelectionChange(Sender);
end;

procedure TRichForm.RightIndMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  FDragging := False;
  RichEdit1.Paragraph.RightIndent := Trunc((Ruler.ClientWidth-RightInd.Left+FDragOfs-2) / RulerAdj)-2*GutterWid;
  SelectionChange(Sender);
end;

procedure TRichForm.CancBtnClick(Sender: TObject);
begin
  ModalResult := mrCancel;
end;

procedure TRichForm.OkBtnClick(Sender: TObject);
begin
  ModalResult := mrOk;
end;

procedure TRichForm.SpeedButton2Click(Sender: TObject);
begin
  VarForm := TVarForm.Create(nil);
  with VarForm do
  if ShowModal = mrOk then
  begin
    ClipBoard.Clear;
    if Variables.ItemIndex >= 0 then
      ClipBoard.AsText := '[' + Variables.Items[Variables.ItemIndex] + ']';
    RichEdit1.PasteFromClipboard;
  end;
  VarForm.Free;
end;

procedure TRichForm.FormActivate(Sender: TObject);
begin
  RichEdit1.SetFocus;
end;


initialization
  RichForm := TRichForm.Create(nil);
  SRichEdit := TRichEdit.Create(nil);
  with SRichEdit do
  begin
    Parent := RichForm;
    Visible := False;
    WordWrap := False;
    Width := 1024;
  end;
  frRegisterObject(TfrRichView,RichForm.Image1.Picture.Bitmap,
    frLoadStr('SInsRichObject'),RichForm);

finalization
  SRichEdit.Free;
  RichForm.Free;
  RichForm := nil;


end.
